import __test__
from luma.migrate.port import ModelPorter
from luma.visual.region import DecisionRegion

from sklearn.datasets import load_digits


X, y = load_digits(return_X_y=True)

port = ModelPorter()
tsne = port.load('example/model/digits_tsne.luma')
ada = port.load('example/model/digits_adaknn.luma')

print(ada.__dict__)

X = tsne.transform()

region = DecisionRegion(estimator=ada,
                        X=X, y=y)

region.plot()



from typing import Any
from luma.interface.super import Transformer, Estimator


__all__ = ['NotFittedError', 'UnsupportedParameterError', 'NotConvergedError',
           'ModelExtensionError']


class NotFittedError(Exception):
    def __init__(self, model: Transformer | Estimator | Any) -> None:
        super().__init__(f"'{model.__class__.__name__}' is not fitted!" + 
                         f" Call '{model.__class__.__name__}.fit()' to fit the model.")
        

class UnsupportedParameterError(Exception):
    def __init__(self, param: Any) -> None:
        super().__init__(f"'{param}' is unsupported!")


class NotConvergedError(Exception):
    def __init__(self, model: Transformer | Estimator | Any) -> None:
        super().__init__(f"'{model.__class__.__name__}' did not converged!" + 
                         f" Try setting '{model.__class__.__name__}.tol' to bigger value.")


class ModelExtensionError(Exception):
    def __init__(self, filename: str) -> None:
        super().__init__(f"'{filename}' is not a '.luma' model file!")




import __test__
from luma.preprocessing.scaler import StandardScaler
from luma.reduction.linear import LDA
from luma.classifier.neighbors import WeightedKNNClassifier
from luma.visual.region import DecisionRegion
from luma.model_selection.split import TrainTestSplit
from luma.pipe.pipeline import Pipeline

from sklearn.datasets import make_blobs


X, y = make_blobs(n_samples=1000, 
                  n_features=4,
                  centers=7, 
                  cluster_std=1.0, 
                  random_state=10)

X_train, X_test, y_train, y_test = TrainTestSplit.split(X, y,
                                                        test_size=0.3,
                                                        random_state=10)

pipe = Pipeline(models=[
                    ('scaler', StandardScaler()),
                    ('lda', LDA()),
                    ('wknn', WeightedKNNClassifier())
                ],
                param_dict={
                    'lda__n_components': 2,
                    'wknn__n_neighbors': 5,
                })

pipe.fit(X_train, y_train)

plot = DecisionRegion(pipe.estimator,
                      *pipe.transform(X_test, y_test))

plot.title = 'Pipeline (StandardScaler, LDA, WKNN)'
plot.title += f'[Score: {pipe.score(X_test, y_test):.4f}]'
plot.plot()



from sklearn.model_selection import train_test_split
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

import __test__
from luma.regressor.neighbors import WeightedKNNRegressor
from luma.metric.regression import RootMeanSquaredError


np.random.seed(42)
X = np.random.uniform(-5, 5, (100, 2))
y = np.sin(X[:, 0]) + np.cos(X[:, 1])

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.3, random_state=42)

model = WeightedKNNRegressor(n_neighbors=5)
model.fit(X_train, y_train)
predictions = model.predict(X_test)

x1_range = np.linspace(X[:, 0].min() - 1, X[:, 0].max() + 1, 50)
x2_range = np.linspace(X[:, 1].min() - 1, X[:, 1].max() + 1, 50)
x1_grid, x2_grid = np.meshgrid(x1_range, x2_range)
grid = np.c_[x1_grid.ravel(), x2_grid.ravel()]

grid_predictions = model.predict(grid).reshape(x1_grid.shape)

fig = plt.figure(figsize=(5.5, 5))
ax = fig.add_subplot(1, 1, 1, projection='3d')

ax.scatter(X_train[:, 0], X_train[:, 1], y_train, 
           color='blue', 
           label='Training Data',
           alpha=0.5)
ax.scatter(X_test[:, 0], X_test[:, 1], y_test, 
           color='red', 
           label='Test Data',
           alpha=0.5)
ax.plot_surface(x1_grid, x2_grid, grid_predictions, 
                cmap='rainbow', 
                label='Predicted Surface',
                alpha=0.5)

ax.set_xlabel(r'$x_1$')
ax.set_ylabel(r'$x_2$')
ax.set_zlabel(r'$y$')
ax.legend()

plt.title(f'Weighted KNN Regressor [RMSE: {model.score(X_test, y_test, RootMeanSquaredError):.4f}]')
plt.tight_layout()
plt.show()




import __test__
from luma.model_selection.split import TrainTestSplit
from luma.preprocessing.scaler import StandardScaler
from luma.classifier.neighbors import KNNClassifier, AdaptiveKNNClassifier
from luma.classifier.logistic import SoftmaxRegressor
from luma.classifier.naive_bayes import GaussianNaiveBayes
from luma.classifier.svm import SVC, KernelSVC
from luma.classifier.tree import DecisionTreeClassifier
from luma.ensemble.forest import RandomForestClassifier

from sklearn.datasets import make_blobs
import matplotlib.pyplot as plt
import numpy as np

seed = 10

X, y = make_blobs(n_samples=500, 
                  centers=7, 
                  cluster_std=1.5, 
                  random_state=seed)

sc = StandardScaler()
X = sc.fit_transform(X)

X_train, X_test, y_train, y_test = TrainTestSplit.split(X, y, 
                                                        test_size=0.3,
                                                        random_state=seed)

models = [KNNClassifier(), AdaptiveKNNClassifier(), SoftmaxRegressor(),
          GaussianNaiveBayes(), SVC(), KernelSVC(), 
          DecisionTreeClassifier(),RandomForestClassifier()]

for model in models:
    model.fit(X_train, y_train)

x1_min, x1_max = X[:, 0].min() - 0.5, X[:, 0].max() + 0.5
x2_min, x2_max = X[:, 1].min() - 0.5, X[:, 1].max() + 0.5
xx1, xx2 = np.meshgrid(np.arange(x1_min, x1_max, 0.02),
                       np.arange(x2_min, x2_max, 0.02))

fig, axes = plt.subplots(2, 4, figsize=(16, 8))

for ax, model in zip(axes.flatten(), models):
    Z = model.predict(np.array([xx1.ravel(), xx2.ravel()]).T)
    Z = Z.reshape(xx1.shape)
    ax.contourf(xx1, xx2, Z, alpha=0.4, cmap='rainbow')
    ax.set_xlim(xx1.min(), xx1.max())
    ax.set_ylim(xx2.min(), xx2.max())

    ax.scatter(X_train[:, 0], X_train[:, 1], 
               c=y_train, cmap='rainbow', marker='o', 
               alpha=0.8, edgecolors='black', label='Train')
    
    ax.scatter(X_test[:, 0], X_test[:, 1], 
               c=y_test, cmap='rainbow', marker='D', 
               alpha=0.8, edgecolors='black', label='Test')
    
    ax.set_xlabel('x')
    ax.set_ylabel('y')
    ax.set_title(f'{model.__class__.__name__} [Score: {model.score(X_test, y_test):.2f}]')
    ax.legend()

plt.tight_layout()
plt.show()



import __test__
from sklearn.datasets import load_iris

from luma.classifier.neighbors import WeightedKNNClassifier
from luma.visual.region import DecisionRegion


iris = load_iris()
X, y = iris.data, iris.target
X = X[:, :2]

wknn = WeightedKNNClassifier()
wknn.fit(X, y)

region = DecisionRegion(estimator=wknn,
                        X=X, 
                        y=y,
                        title='Weighted KNN Classifier')

region.plot()



import __test__
from luma.regressor.neighbors import AdaptiveKNNRegressor
from luma.metric.regression import RootMeanSquaredError as rmse

from sklearn.datasets import make_regression
import matplotlib.pyplot as plt
import numpy as np


X, y = make_regression(n_features=1, n_samples=100, noise=10, random_state=0)

adaknn = AdaptiveKNNRegressor(n_density=10,
                              min_neighbors=5,
                              max_neighbors=20)

adaknn.fit(X, y)

X_test = np.linspace(-3, 3, 100).reshape(-1, 1)
y_pred = adaknn.predict(X_test)

plt.scatter(X, y, c='royalblue', marker='x', s=15, label='Train values')
plt.plot(X_test, y_pred, color='crimson', linewidth=2, label='Predicted plot')
plt.grid()
plt.legend()
plt.xlabel('x')
plt.ylabel('y')
plt.title(f'Adaptive kNN Regression [RMSE: {adaknn.score(X, y, metric=rmse):.4f}]')
plt.tight_layout()
plt.show()



from luma.migrate.port import ModelPorter

from sklearn.datasets import load_digits
import matplotlib.pyplot as plt


X, y = load_digits(return_X_y=True)


port = ModelPorter()
tsne = port.load(filepath='example/model/digits_tsne.luma')

X = tsne.transform()

plt.scatter(X[:, 0], X[:, 1], c=y, cmap='rainbow')
plt.tight_layout()
plt.show()
